////////////////////////////////////////////////////////////////
// FILE:      scenario.cpp
// CONTAINS:  various display tests
////////////////////////////////////////////////////////////////

#include "os.h"
#include "GL/gl.h"
#include "GL/glut.h"
#include "lib_defs.h"
#include "mygl.h"

#include "objreader.h"
#include <iostream>

extern void myBindTexture(char *name);
extern IMAGE *CurrTex;
extern int Grid;
extern void DrawPixelGrid();
extern void init_zbuf();

using namespace std;


////////////////////////////////////////////////////////////////
// FUNC:  scenario_A()
// DOES:  basic test of transformations: modelview + proj + /h + viewport
////////////////////////////////////////////////////////////////

void scenario_A()
{
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();

	myBegin(GL_POINTS);
	
	myColor(0,0,1);       // blue
	myVertex(0,0,0);      // center of screen in NDCS
	
	myColor(1,0,0);       // red 
	myVertex(0.5,0,0);    // off to the right
	
	myColor(0,1,0);       // green
	myVertex(0.5,0.5,0);  // off to the right and up
	
	myEnd();
	
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_B()
// DOES:  basic test of line drawing
////////////////////////////////////////////////////////////////

void scenario_B()
{
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();

	float pi = 3.1415926536;

	for (int i=0; i<10; ++i) {
        myBegin(GL_LINE_LOOP);	
			myColor(1,0,0);       
			myVertex(0, 0, 0);  
			myColor(0,0,1);       
			myVertex(cos(i*2*pi/10), sin(i*2*pi/10), 0);
		myEnd();
	}
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_C()
// DOES:  basic test of viewport transformation
////////////////////////////////////////////////////////////////

void scenario_C()
{
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();

	myColor(1,0,0);
	myBegin(GL_TRIANGLES);
	myVertex(0,0,0);
	myVertex(0.5,0,0);
	myVertex(0.5,0.5,0);	
	myEnd();

	myColor(0,1,1);
	myBegin(GL_TRIANGLES);
	myVertex(0,0,0);
	myVertex(0.5,0,0);
	myVertex(0.5,-0.5,0);	
	myEnd();



	myBegin(GL_POINTS);
	myColor(1,0,0);       // red
	myVertex(0,0,0);      // center of screen in NDCS
	myColor(0,1,0);       // green 
	myVertex(0.5,0,0);    // off to the right
	myColor(0,0,1);       // blue
	myVertex(0.5,0.5,0);  // off to the right and up
	myEnd();

	myColor(0,0,1);
	myBegin(GL_TRIANGLES);
	myVertex(-0.5,0.5,0);
	myVertex(-0.5,-0.5,0);
	myVertex(-1.5,0,0);
	
	myEnd();
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_D()
// DOES:  test of colour interpolation
////////////////////////////////////////////////////////////////

void scenario_D()
{
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();

	myBegin(GL_TRIANGLES);
	myColor(1,0,0);       // red
	myVertex(0,0,0);      // center of screen in NDCS
	myColor(0,1,0);       // green 
	myVertex(0.6,0,0);    // off to the right
	myColor(0,0,1);       // blue
	myVertex(0.6,0.6,0);  // off to the right and up
	myEnd();
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_E()
// DOES:  test translation, rotate, and scale
////////////////////////////////////////////////////////////////

void scenario_E()
{
	scenario_D();
	myTranslate(0,-1,0);
	myRotate(90,0,0,1);
	myScale(2,2,2);

	myBegin(GL_TRIANGLES);
	myColor(1,0,0);       // red
	myVertex(0,0,0);      // center of screen in NDCS
	myColor(0,1,0);       // green 
	myVertex(0.5,0,0);    // off to the right
	myColor(0,0,1);       // blue
	myVertex(0.5,0.5,0);  // off to the right and up
	myEnd();
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_F()
// DOES:  test of viewing and projection transformations
////////////////////////////////////////////////////////////////

void scenario_F()
{
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myFrustum(-0.5,0.5,-0.5,0.5,1,10);
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();
	myLookAt(1,0,1,0,0,0,0,1,0);

	myBegin(GL_TRIANGLES);
	myColor(1,0,0);       // red
	myVertex(0,0,0);      // center of screen in NDCS
	myColor(0,1,0);       // green 
	myVertex(0.5,0,0);    // off to the right
	myColor(0,0,1);       // blue
	myVertex(0.5,0.5,0);  // off to the right and up
	myEnd();
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_G()
// DOES:  test of Z-buffer
////////////////////////////////////////////////////////////////

void scenario_G()
{
	for (int n=0; n<100; n++) {
		glClear( GL_COLOR_BUFFER_BIT );
		if (Grid) DrawPixelGrid();
		init_zbuf();

		myMatrixMode(GL_PROJECTION);
		myLoadIdentity();
		myMatrixMode(GL_MODELVIEW);
		myLoadIdentity();

		myRotate((n>50?100-n:n),1,1,1);

		myColor(0,1,0);       // green
		myBegin(GL_TRIANGLES);
		myVertex(-1,1,1);      // center of screen in NDCS
		myVertex(1,1,1);    // off to the right
		myVertex(0,-1,-1);  // off to the right and up
		myEnd();

		myScale(0.5,0.5,0.5);
		myColor(1,0,0);       // red
		myBegin(GL_TRIANGLES);
		myVertex(-1,-1,0);      // center of screen in NDCS
		myVertex(1,-1,0);    // off to the right
		myVertex(1,1,0);  // off to the right and up
		myEnd();
		myBegin(GL_TRIANGLES);
		myVertex(-1,-1,0);      // center of screen in NDCS
		myVertex(1,1,0);    // off to the right
		myVertex(-1,1,0);  // off to the right and up
		myEnd();

		glutSwapBuffers();
	}
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_H()
// DOES:  test of texture mapping
////////////////////////////////////////////////////////////////

void scenario_H()
{
	for (int n=0; n<40; n++) {
		glClear( GL_COLOR_BUFFER_BIT );
		if (Grid) DrawPixelGrid();
		init_zbuf();

		myMatrixMode(GL_PROJECTION);
		myLoadIdentity();
		myFrustum(-0.5,0.5,-0.5,0.5,1,10);
		myMatrixMode(GL_MODELVIEW);
		myLoadIdentity();
		myLookAt(2,0,2,0,0,0,0,1,0);

		myRotate((n>20?40-n:n),1,1,1);		

		myBindTexture("clocktower.ppm");

		myBegin(GL_TRIANGLES);
		myTexCoord(0,1);
		myVertex(-1,-1,0);      // center of screen in NDCS
		myTexCoord(1,1);
		myVertex(1,-1,0);    // off to the right
		myTexCoord(1,0);
		myVertex(1,1,0);  // off to the right and up
		myEnd();

		myBegin(GL_TRIANGLES);
		myTexCoord(0,1);
		myVertex(-1,-1,0);      // center of screen in NDCS
		myTexCoord(1,0);
		myVertex(1,1,0);    // off to the right
		myTexCoord(0,0);
		myVertex(-1,1,0);  // off to the right and up
		myEnd();

		glutSwapBuffers();

	}
	
	CurrTex = (IMAGE *) 0;    // disable textures
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_I()
// DOES:  Uses objReader to draw a mesh
////////////////////////////////////////////////////////////////

void scenario_I()
{
	objReader r;
	r.loadObj("cube.obj");
	r.centerObject();
	r.resizeObject();
	
	glClear( GL_COLOR_BUFFER_BIT );
	myMatrixMode(GL_PROJECTION);
	myLoadIdentity();
	myMatrixMode(GL_MODELVIEW);
	myLoadIdentity();
	myScale(0.5, 0.5, 0.5);
	myRotate(45,1,1,1);

	myBegin(GL_LINE_LOOP);
	r.displayObj();	
	myEnd();
}

////////////////////////////////////////////////////////////////
// FUNC:  scenario_J()
// DOES:  your own scene
////////////////////////////////////////////////////////////////

void scenario_J()
{
}
